// Temperature Controlled Fan
// Written by Theron Wierenga December 2017
// For use with Ultimaker 2+ 3D printer

#define FAN 2
#define UP A2
#define DOWN A3

// Include the BME280
#include "SparkFunBME280.h"
BME280 mySensor;
#include "Wire.h"
// Include the LCD display
#include <LiquidCrystal_I2C.h>
// Some display have an address of 0x27, this one is 0x3F
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address
// The variables to store settings and temperature
float temp_high = 28.0, temp_low = 27.0, current_temp, current_tempf;

void setup() {

  // Serial.begin(115200);
  // Setup the pins needed
  pinMode(UP, INPUT_PULLUP);
  pinMode(DOWN, INPUT_PULLUP);
  pinMode(FAN, OUTPUT);
  // Start the LCD
  lcd.begin(20, 4);
  // Start the BME280 and calibrate it
  mySensor.settings.commInterface = I2C_MODE;
  // The I2C address may cary with different BME280 boards
  mySensor.settings.I2CAddress = 0x76;
  mySensor.settings.runMode = 3; //Normal mode
  mySensor.settings.tStandby = 0;
  mySensor.settings.filter = 0;
  mySensor.settings.tempOverSample = 1;
  mySensor.settings.pressOverSample = 1;
  mySensor.settings.humidOverSample = 1;
  mySensor.begin();
  mySensor.calibration.dig_T1;
  mySensor.calibration.dig_T2;
  mySensor.calibration.dig_T3;
  mySensor.calibration.dig_P1;
  mySensor.calibration.dig_P2;
  mySensor.calibration.dig_P3;
  mySensor.calibration.dig_P4;
  mySensor.calibration.dig_P5;
  mySensor.calibration.dig_P6;
  mySensor.calibration.dig_P7;
  mySensor.calibration.dig_P8;
  mySensor.calibration.dig_P9;
  mySensor.calibration.dig_H1;
  mySensor.calibration.dig_H2;
  mySensor.calibration.dig_H3;
  mySensor.calibration.dig_H4;
  mySensor.calibration.dig_H5;
  mySensor.calibration.dig_H6;
}

void loop() {

//  Serial.print("Temperature: ");
//  Serial.print(mySensor.readTempC(), 2);
//  Serial.println(" degrees C");
//  
//  Serial.print(mySensor.readTempC()*9.0/5.0 + 32.0, 2);
//  Serial.println(" degrees F");
//
//  Serial.print("Temperature: ");
//  Serial.print(mySensor.readTempF());
//  Serial.println(" degrees F");

  // Write the high and low temperature settings to the LCD
  lcd.setCursor(0, 0);
  lcd.print("High = ");
  lcd.print(temp_high);
  lcd.setCursor(0, 1);
  lcd.print("Low  = ");
  lcd.print(temp_low);

  // Write the current temperature to the LCD
  lcd.setCursor(0, 2);
  lcd.print("Temp = ");
  current_temp = mySensor.readTempC();
  lcd.print(current_temp);

  // Add the Fahrenheit reading after the Celsius
  current_tempf = mySensor.readTempF();
  lcd.setCursor(14, 2);
  lcd.print(current_tempf);
  lcd.print(" ");

  // Check if temperature is above the high set point
  if (current_temp >= temp_high)  // Turn fan on
  {
      digitalWrite(FAN, HIGH);
      lcd.setCursor(0, 3);
      lcd.print("Fan is on ");
  }

  // Check if temperature is below the low set point
  if (current_temp <= temp_low)  // Turn fan off
  {
      digitalWrite(FAN, LOW);
      lcd.setCursor(0, 3);
      lcd.print("Fan is off");
  }

  // Check if up button pressed
  if (digitalRead(UP) == 0)
  {
      temp_high = temp_high + 0.1;
      // temp_low can be changed for greater separation between high and low values
      temp_low = temp_high - 1.0;
  }

  // Check if low button pressed
  if (digitalRead(DOWN) == 0)
  {
      temp_high = temp_high - 0.1;
      temp_low = temp_high - 1.0;
  }
  
  delay(100);
}
